from typing import Any
from typing import Dict


async def sig_gather(hub, profile: Dict[str, Any]):
    """
    Get the metadata for the given profile.
    The profile contains the login information for a provider, similar to ctx.acct in idem states
    """



CLI_CONFIG = {
    "input_file": {
        "positional": True,
        "subcommands": ["encrypt", "decrypt"],
    },
    "acct_key": {"subcommands": ["encrypt", "decrypt"]},
    "acct_file": {"subcommands": ["encrypt", "decrypt"]},
    "extras": {},
    "render_pipe": {},
    "crypto_plugin": {"subcommands": ["encrypt", "decrypt"]},
    "output": {"source": "rend", "subcommands": ["decrypt"]},
    "output_file": {"subcommands": ["encrypt"]},
}
CONFIG = {
    "acct_file": {
        "os": "ACCT_FILE",
        "default": None,
        "help": "An encrypted acct file",
    },
    "acct_key": {
        "os": "ACCT_KEY",
        "default": None,
        "help": "The key to encrypt the file with, if no key is passed a key will be generated and displayed on after the encrypted file is created",
    },
    "extras": {
        "render": "json",
        "default": {},
        "help": "Additional arbitrary configuration values as a json string",
    },
    "crypto_plugin": {
        "default": "fernet",
        "help": "The crypto plugin to use with acct",
    },
    "render_pipe": {
        "default": "jinja|yaml",
        "help": "The render pipe to use to parse input files before they are encrypted",
    },
    "serial_plugin": {
        "default": "msgpack",
        "help": "The pop-serial plugin to use to serialize data before it is encrypted",
    },
    "allowed_backend_profiles": {
        "default": None,
        "help": "A list of backend profile names to allow, defaults to ALL",
    },
    "output_file": {"default": None, "help": "The output file name when encrypting"},
}
SUBCOMMANDS = {
    "encrypt": {x: "Use the acct command to encrypt data" for x in ("desc", "help")},
    "decrypt": {x: "Use the acct command to decrypt data" for x in ("desc", "help")},
}
DYNE = {
    "acct": ["acct"],
    "crypto": ["crypto"],
}



from typing import Dict

import cryptography.fernet


def generate_key(hub) -> str:
    key = cryptography.fernet.Fernet.generate_key()
    return key.decode("utf-8")


def encrypt(hub, data: Dict, key: str) -> bytes:
    fernet = cryptography.fernet.Fernet(key)
    raw = hub.crypto.init.dump(data)
    return fernet.encrypt(raw)


def decrypt(hub, data: bytes, key: str) -> Dict:
    fernet = cryptography.fernet.Fernet(key)
    raw = fernet.decrypt(data)
    return hub.crypto.init.load(raw)



import binascii
import os
from typing import Dict

from cryptography.hazmat.primitives.ciphers.aead import AESGCM


def generate_key(hub) -> str:
    """
    This is a version of `AESGCM.generate_key()` that generates a key
    that can be stored in an environment variable.
    """
    return binascii.b2a_base64(os.urandom(256 // 8)).decode("utf-8").strip()


def encrypt(hub, data: Dict, key: str) -> bytes:
    key = binascii.a2b_base64(key.encode("utf-8"))
    nonce = os.urandom(12)
    aesgcm = AESGCM(key)
    raw = hub.crypto.init.dump(data)
    encrypted = aesgcm.encrypt(nonce=nonce, data=raw, associated_data=None)
    # An encrypted file is returned with the nonce as a header
    return nonce + encrypted


def decrypt(hub, data: bytes, key: str) -> Dict:
    key = binascii.a2b_base64(key.encode("utf-8"))
    # The nonce is derived from the header
    nonce = data[:12]
    data = data[12:]
    aesgcm = AESGCM(key)
    raw = aesgcm.decrypt(nonce=nonce, data=data, associated_data=None)
    return hub.crypto.init.load(raw)



from typing import Any
from typing import Dict
from typing import List


async def gather(
    hub,
    profile: Dict[str, Any],
    providers: List[str],
    hard_fail: bool = False,
):
    """
    Get the metadata for the given profile, which probably comes from an idem state's ctx.acct

    :param hub:
    :param profile: A dictionary containing login credentials for the provider
    :param providers: A list of providers that may be related to this sub
    :param hard_fail: Raise exceptions instead of catching them
    """
    for provider in providers:
        if provider in hub.acct.metadata:
            try:
                return await hub.acct.metadata[provider].gather(profile)
            except Exception as e:
                hub.log.error(
                    f"Error gathering metadata from {provider}: {e.__class__.__name__}: {e}"
                )
                if hard_fail:
                    raise
    else:
        for provider in providers:
            hub.log.debug(
                f"No metadata gather function found in 'hub.acct.metadata.{provider}.gather'"
            )
        return {}
